﻿using Microsoft.AspNetCore.Mvc.ApplicationParts;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.IO;
using System.Reflection;

namespace Falcon.SugarApi.Plugin.Service
{
    /// <summary>
    /// 服务扩展 服务端调用
    /// </summary>
    public static class IServiceCollectionExtend
    {
        /// <summary>
        /// 
        /// </summary>
        //public static List<Assembly> Plugins { get; set; } = new List<Assembly>();

        /// <summary>
        /// 注册插件。
        /// <para>查找插件中的IServicePlugin实现，并运行其中的AddServices方法注册和初始化插件</para>
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="config">应用程序配置</param>
        /// <param name="pluginNames">插件名称数组</param>
        /// <exception cref="FileNotFoundException"></exception>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddPluginService(this IServiceCollection services,IConfiguration config,params string[] pluginNames) {
            var name = typeof(IServicePlugin).FullName;
            if(name == null) {
                return services;
            }
            var basePath = AppDomain.CurrentDomain.BaseDirectory;
            foreach(var pn in pluginNames) {
                var pf = pn.EndsWith(".dll") ? pn : pn + ".dll";
                if(!Path.IsPathRooted(pf)) {
                    foreach(var path in PluginOptions.PluginPaths) {
                        var tfp = Path.Combine(basePath,path,pf);
                        if(File.Exists(tfp)) {
                            pf = tfp;
                            break;
                        }
                    }
                }
                if(!File.Exists(pf)) {
                    throw new FileNotFoundException(pf);
                }
                //var pa = AssemblyLoadHelp.ALCLoad(pf);
                var pa = AssemblyLoadHelp.AssemblyLoad(pf);

                if(pa != null) {
                    services.AddPluginService(config,pa);
                }
            }
            return services;
        }

        /// <summary>
        /// 注册插件。
        /// <para>查找插件中的IServicePlugin实现，并运行其中的AddServices方法注册和初始化插件</para>
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="config">应用程序配置</param>
        /// <param name="plugins">插件程序集数组</param>
        /// <exception cref="PluginExecAddServicesException"></exception>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddPluginService(this IServiceCollection services,IConfiguration config,params Assembly[] plugins) {
            var name = typeof(IServicePlugin).FullName;
            if(name == null) {
                return services;
            }
            foreach(Assembly ass in plugins) {
                foreach(Type type in ass.GetTypes()) {
                    if(type == null || !type.IsPublic || type.GetInterface(name) == null) {
                        continue;
                    }
                    var tname = type.FullName;
                    if(tname == null) {
                        continue;
                    }

                    if(type.Assembly.CreateInstance(tname) is not IServicePlugin obj) {
                        continue;
                    }
                    try {
                        obj.AddServices(services,config);
                    }
                    catch(Exception ex) {
                        throw new PluginExecAddServicesException(type,ex);
                    }
                }
            }
            return services;
        }

        /// <summary>
        /// 注册插件中的控制器。一般由插件自行注册。
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="assemblyPath">插件程序集文件路径</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddPluginsController(this IServiceCollection services,string assemblyPath) {
            var pluginAssembly = Assembly.LoadFrom(assemblyPath);
            return services.AddPluginsController(pluginAssembly);
        }

        /// <summary>
        /// 注册插件中的控制器。一般由插件自行注册。
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="assembly">插件程序集</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddPluginsController(this IServiceCollection services,Assembly assembly) {
            var part = new AssemblyPart(assembly);
            services.AddControllers().ConfigureApplicationPartManager(apm => {
                if(!apm.ApplicationParts.Contains(part)) {
                    apm.ApplicationParts.Add(part);
                }
            });
            return services;
        }

    }
}
